@@ -1,8 +1,8 @@
-/* $Id: upnpdescgen.c,v 1.80 2015/12/12 09:10:29 nanard Exp $ */
+/* $Id: upnpdescgen.c,v 1.82 2016/02/16 12:15:02 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2015 Thomas Bernard
+ * (c) 2006-2016 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -24,6 +24,7 @@
 
 
 /* Event magical values codes */
+#define SETUPREADY_MAGICALVALUE (248)
 #define CONNECTIONSTATUS_MAGICALVALUE (249)
 #define FIREWALLENABLED_MAGICALVALUE (250)
 #define INBOUNDPINHOLEALLOWED_MAGICALVALUE (251)
@@ -47,6 +48,10 @@ static const char * const upnpdefaultvalues[] =
 	0,
 	"IP_Routed"/*"Unconfigured"*/, /* 1 default value for ConnectionType */
 	"3600", /* 2 default value for PortMappingLeaseDuration */
+	"Unconfigured",	/* 3 default value for ConnectionStatus */
+	"0", /* 4 default value for RSIPAvailable */
+	"1", /* 5 default value for NATEnabled */
+	"ERROR_NONE", /* 6 default value for LastConnectionError */
 };
 
 static const char * const upnpallowedvalues[] =
@@ -117,11 +122,13 @@ static const char * magicargname[] = {
 	"InternalClient",	/* 5 */
 	"InternalPort",		/* 6 */
 	"IsWorking",		/* 7 */
+#ifdef ENABLE_DP_SERVICE
 	"ProtocolType",		/* 8 */
 	"InMessage",		/* 9 */
 	"OutMessage",		/* 10 */
 	"ProtocolList",		/* 11 */
-	"RoleList"			/* 12 */
+	"RoleList",			/* 12 */
+#endif /* ENABLE_DP_SERVICE */
 };
 
 static const char xmlver[] =
@@ -230,9 +237,9 @@ static const struct XMLElt rootDesc[] =
 			"urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"},
 	/*{"/serviceId", "urn:upnp-org:serviceId:WANCommonInterfaceConfig"}, */
 	{"/serviceId", "urn:upnp-org:serviceId:WANCommonIFC1"}, /* required */
+	{"/SCPDURL", WANCFG_PATH},
 	{"/controlURL", WANCFG_CONTROLURL},
 	{"/eventSubURL", WANCFG_EVENTURL},
-	{"/SCPDURL", WANCFG_PATH},
 /* 38 */
 	{"device", INITHELPER(39,12)},
 /* 39 */
@@ -261,16 +268,16 @@ static const struct XMLElt rootDesc[] =
 		/* urn:schemas-upnp-org:service:WANIPConnection:2 for v2 */
 	{"/serviceId", SERVICE_ID_WANIPC},
 		/* urn:upnp-org:serviceId:WANIPConn1 or 2 */
+	{"/SCPDURL", WANIPC_PATH},
 	{"/controlURL", WANIPC_CONTROLURL},
 	{"/eventSubURL", WANIPC_EVENTURL},
-	{"/SCPDURL", WANIPC_PATH},
 #ifdef ENABLE_6FC_SERVICE
 /* 58 */
 	{"/serviceType", "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1"},
 	{"/serviceId", "urn:upnp-org:serviceId:WANIPv6Firewall1"},
+	{"/SCPDURL", WANIP6FC_PATH},
 	{"/controlURL", WANIP6FC_CONTROLURL},
 	{"/eventSubURL", WANIP6FC_EVENTURL},
-	{"/SCPDURL", WANIP6FC_PATH},
 #endif
 /* 58 / 63 = SERVICES_OFFSET*/
 #if defined(HAS_DUMMY_SERVICE) || defined(ENABLE_L3F_SERVICE) || defined(ENABLE_DP_SERVICE)
@@ -281,17 +288,17 @@ static const struct XMLElt rootDesc[] =
 /* 60 / 65 = SERVICES_OFFSET+2 */
 	{"/serviceType", "urn:schemas-dummy-com:service:Dummy:1"},
 	{"/serviceId", "urn:dummy-com:serviceId:dummy1"},
+	{"/SCPDURL", DUMMY_PATH},
 	{"/controlURL", "/dummy"},
 	{"/eventSubURL", "/dummy"},
-	{"/SCPDURL", DUMMY_PATH},
 #endif
 #ifdef ENABLE_L3F_SERVICE
 /* 60 / 65 = SERVICES_OFFSET+2 */
 	{"/serviceType", "urn:schemas-upnp-org:service:Layer3Forwarding:1"},
 	{"/serviceId", "urn:upnp-org:serviceId:L3Forwarding1"},
+	{"/SCPDURL", L3F_PATH},
 	{"/controlURL", L3F_CONTROLURL}, /* The Layer3Forwarding service is only */
 	{"/eventSubURL", L3F_EVENTURL}, /* recommended, not mandatory */
-	{"/SCPDURL", L3F_PATH},
 #endif
 #ifdef ENABLE_DP_SERVICE
 /* InternetGatewayDevice v2 :
@@ -302,9 +309,9 @@ static const struct XMLElt rootDesc[] =
 /* 65 / 70 = SERVICES_OFFSET+7 */
 	{"/serviceType", "urn:schemas-upnp-org:service:DeviceProtection:1"},
 	{"/serviceId", "urn:upnp-org:serviceId:DeviceProtection1"},
+	{"/SCPDURL", DP_PATH},
 	{"/controlURL", DP_CONTROLURL},
 	{"/eventSubURL", DP_EVENTURL},
-	{"/SCPDURL", DP_PATH},
 #endif
 	{0, 0}
 };
@@ -486,24 +493,30 @@ static const struct action WANIPCnActions[] =
 /* ignore "warning: missing initializer" */
 #pragma GCC diagnostic ignored "-Wmissing-field-initializers"
 
+/* struct stateVar :
+ * {name, itype(&event), idefault, iallowedlist, ieventvalue} */
 static const struct stateVar WANIPCnVars[] =
 {
 /* 0 */
 #if 0
 	{"ConnectionType", 0, 0/*1*/}, /* required */
 	{"PossibleConnectionTypes", 0|0x80, 0, 14, 15},
-#endif
+#elif 0
 	{"ConnectionType", 0, 1, 14, 15}, /* required */
 	{"PossibleConnectionTypes", 0|0x80, 0, 0, 15},
+#else
+	{"ConnectionType", 0, 1, 0, 15}, /* required */
+	{"PossibleConnectionTypes", 0|0x80, 0, 14, 15},
+#endif
 	 /* Required
 	  * Allowed values : Unconfigured / IP_Routed / IP_Bridged */
-	{"ConnectionStatus", 0|0x80, 0/*1*/, 18,
+	{"ConnectionStatus", 0|0x80, 3, 18,
 	 CONNECTIONSTATUS_MAGICALVALUE }, /* required */
 	 /* Allowed Values : Unconfigured / Connecting(opt) / Connected
 	  *                  PendingDisconnect(opt) / Disconnecting (opt)
 	  *                  Disconnected */
 	{"Uptime", 3, 0},	/* Required */
-	{"LastConnectionError", 0, 0, 25},	/* required : */
+	{"LastConnectionError", 0, 6, 25},	/* required : */
 	 /* Allowed Values : ERROR_NONE(req) / ERROR_COMMAND_ABORTED(opt)
 	  *                  ERROR_NOT_ENABLED_FOR_INTERNET(opt)
 	  *                  ERROR_USER_DISCONNECT(opt)
@@ -513,8 +526,8 @@ static const struct stateVar WANIPCnVars[] =
 	  *                  ERROR_NO_CARRIER(opt)
 	  *                  ERROR_IP_CONFIGURATION(opt)
 	  *                  ERROR_UNKNOWN(opt) */
-	{"RSIPAvailable", 1, 0}, /* required */
-	{"NATEnabled", 1, 0},    /* required */
+	{"RSIPAvailable", 1, 4}, /* required */
+	{"NATEnabled", 1, 5},    /* required */
 	{"ExternalIPAddress", 0|0x80, 0, 0,
 	 EXTERNALIPADDRESS_MAGICALVALUE}, /* required. Default : empty string */
 	{"PortMappingNumberOfEntries", 2|0x80, 0, 0,
@@ -752,21 +765,21 @@ static const struct serviceDesc scpd6FC =
 
 static const struct argument SendSetupMessageArgs[] =
 {
-	{1|0x80|(8<<2), 6},	/* ProtocolType */
-	{1|0x80|(9<<2), 5},	/* InMessage */
-	{2|0x80|(10<<2), 5},	/* OutMessage */
+	{1|0x80|(8<<2), 6},		/* ProtocolType : in ProtocolType / A_ARG_TYPE_String */
+	{1|0x80|(9<<2), 5},		/* InMessage : in InMessage / A_ARG_TYPE_Base64 */
+	{2|0x80|(10<<2), 5},	/* OutMessage : out OutMessage / A_ARG_TYPE_Base64 */
 	{0, 0}
 };
 
 static const struct argument GetSupportedProtocolsArgs[] =
 {
-	{2|0x80|(11<<2), 1},	/* ProtocolList */
+	{2|0x80|(11<<2), 1},	/* ProtocolList : out ProtocolList / SupportedProtocols */
 	{0, 0}
 };
 
 static const struct argument GetAssignedRolesArgs[] =
 {
-	{2|0x80|(12<<2), 6},	/* RoleList */
+	{2|0x80|(12<<2), 6},	/* RoleList : out RoleList / A_ARG_TYPE_String */
 	{0, 0}
 };
 
@@ -780,7 +793,7 @@ static const struct action DPActions[] =
 
 static const struct stateVar DPVars[] =
 {
-	{"SetupReady", 1|0x80},
+	{"SetupReady", 1|0x80, 0, 0, SETUPREADY_MAGICALVALUE},
 	{"SupportedProtocols", 0},
 	{"A_ARG_TYPE_ACL", 0},
 	{"A_ARG_TYPE_IdentityList", 0},
@@ -939,6 +952,13 @@ genXML(char * str, int * len, int * tmplen,
 			/*printf("<%s>\n", eltname); */
 			str = strcat_char(str, len, tmplen, '<');
 			str = strcat_str(str, len, tmplen, eltname);
+			if(memcmp(eltname, "root ", 5) == 0) {
+				char configid_str[16];
+				/* add configId attribute, required by UDA 1.1 */
+				snprintf(configid_str, sizeof(configid_str), "\"%u\"", upnp_configid);
+				str = strcat_str(str, len, tmplen, " configId=");
+				str = strcat_str(str, len, tmplen, configid_str);
+			}
 			str = strcat_char(str, len, tmplen, '>');
 			k = (unsigned long)p[i].data;
 			i = k & 0xffff;
@@ -1092,6 +1112,14 @@ genServiceDesc(int * len, const struct serviceDesc * s)
 		str = strcat_str(str, len, &tmplen, "</name><dataType>");
 		str = strcat_str(str, len, &tmplen, upnptypes[vars[i].itype & 0x0f]);
 		str = strcat_str(str, len, &tmplen, "</dataType>");
+		/*if(vars[i].defaultValue) */
+		if(vars[i].idefault)
+		{
+		  str = strcat_str(str, len, &tmplen, "<defaultValue>");
+		  /*str = strcat_str(str, len, &tmplen, vars[i].defaultValue); */
+		  str = strcat_str(str, len, &tmplen, upnpdefaultvalues[vars[i].idefault]);
+		  str = strcat_str(str, len, &tmplen, "</defaultValue>");
+		}
 		if(vars[i].iallowedlist)
 		{
 		  if((vars[i].itype & 0x0f) == 0)
@@ -1114,14 +1142,6 @@ genServiceDesc(int * len, const struct serviceDesc * s)
 		    str = strcat_str(str, len, &tmplen, "</maximum></allowedValueRange>");
 		  }
 		}
-		/*if(vars[i].defaultValue) */
-		if(vars[i].idefault)
-		{
-		  str = strcat_str(str, len, &tmplen, "<defaultValue>");
-		  /*str = strcat_str(str, len, &tmplen, vars[i].defaultValue); */
-		  str = strcat_str(str, len, &tmplen, upnpdefaultvalues[vars[i].idefault]);
-		  str = strcat_str(str, len, &tmplen, "</defaultValue>");
-		}
 		str = strcat_str(str, len, &tmplen, "</stateVariable>");
 		/*str = strcat_char(str, len, &tmplen, '\n'); // TEMP ! */
 		i++;
@@ -1195,6 +1215,13 @@ genEventVars(int * len, const struct serviceDesc * s)
 			switch(v->ieventvalue) {
 			case 0:
 				break;
+#ifdef ENABLE_DP_SERVICE
+			case SETUPREADY_MAGICALVALUE:
+				/* always ready for setup */
+				snprintf(tmp, sizeof(tmp), "%d", 1);
+				str = strcat_str(str, len, &tmplen, tmp);
+				break;
+#endif
 			case CONNECTIONSTATUS_MAGICALVALUE:
 				/* or get_wan_connection_status_str(ext_if_name) */
 				str = strcat_str(str, len, &tmplen,
